@@ -23,7 +23,6 @@ from __future__ import print_function, division
 import numpy as np
 from numpy import log, log1p, exp, logaddexp
 import math
-import operator
 import sys
 from .utils import resample_equal
 from .ordertest import UniformOrderAccumulator
@@ -41,9 +40,8 @@ class TreeNode(object):
             used to order nodes
         id: int
             refers to the order of discovery and storage (PointPile)
-        children: list of nodes
+        children: list of :class:TreeNode objects
             children nodes. if None, a empty list is used.
-
         """
         self.value = value
         self.id = id
@@ -91,9 +89,10 @@ class BreadthFirstIterator(object):
 
         Returns
         --------
-        None if done.
-        rootid, node, (active_nodes, active_root_ids, active_node_values, active_node_ids)
-        otherwise
+        tuple or None:
+            None if done.
+            `rootid, node, (active_nodes, active_root_ids, active_node_values, active_node_ids)`
+            otherwise
 
         """
         if self.active_nodes == []:
@@ -121,9 +120,14 @@ class BreadthFirstIterator(object):
         assert len(self.active_nodes) == len(self.active_node_values)
 
     def expand_children_of(self, rootid, node):
-        """Replace the current node with its children.
+        """Replace the current node with its children in the iterators list of active nodes.
 
-        rootid and node have to come from the most recent call to next_node.
+        Parameters
+        ----------
+        rootid: int
+            index of the root returned by the most recent call to :pyfunc:BreadthFirstIterator.next_node
+        node: :pyclass:TreeNode
+            node returned by the most recent call to :pyfunc:BreadthFirstIterator.next_node
         """
         # print("replacing %.1f" % node.value, len(node.children))
         i = self.next_index
@@ -158,12 +162,18 @@ class BreadthFirstIterator(object):
 
 
 def _stringify_lanes(lanes, char='║'):
-    """ unicode-draw lanes, fill with vertical stripes or spaces """
+    """unicode-draw lanes, fill with vertical stripes or spaces."""
     return ''.join([' ' if n is None else char for n in lanes])
 
 
 def print_tree(roots, title='Tree:'):
-    """Print a pretty yet compact graphic of the tree."""
+    """Print a pretty yet compact graphic of the tree.
+
+    Parameters
+    ----------
+    roots: list of :pyclass:TreeNode
+        tree
+    """
     print()
     print(title)
     explorer = BreadthFirstIterator(roots)
@@ -206,7 +216,17 @@ def print_tree(roots, title='Tree:'):
 
 
 def dump_tree(filename, roots, pointpile):
-    """Write a copy of the tree to a HDF5 file."""
+    """Write a copy of the tree to a HDF5 file.
+
+    Parameters
+    ----------
+    filename: str
+        output filename
+    roots: list of :pyclass:TreeNode
+        tree to store
+    pointpile: :class:PointPile
+        information on the node points
+    """
     import h5py
 
     nodes_from_ids = []
@@ -235,7 +255,20 @@ def dump_tree(filename, roots, pointpile):
 
 
 def count_tree(roots):
-    """Return the maximum number of parallel edges and the total number of nodes."""
+    """Return the total number of nodes and maximum number of parallel edges.
+
+    Parameters
+    ----------
+    roots: list of :pyclass:TreeNode
+        tree
+
+    Returns
+    --------
+    count: int
+        total number of nodes
+    maxwidth: int
+        maximum number of active/parallel nodes encountered
+    """
     explorer = BreadthFirstIterator(roots)
     nnodes = 0
     maxwidth = 0
@@ -251,14 +284,26 @@ def count_tree(roots):
 
 
 def count_tree_between(roots, lo, hi):
-    """Build basic tree statistics.
+    """Compute basic statistics about a tree.
+
+    Return the total number of nodes and maximum number of parallel edges,
+    but only considering a interval of the tree.
+
+    Parameters
+    ----------
+    roots: list of :pyclass:TreeNode
+        tree
+    lo: float
+        lower value threshold
+    hi: float
+        upper value threshold
 
     Returns
     --------
     nnodes: int
-        the total number of nodes in the value interval lo .. hi (inclusive).
+        total number of nodes in the value interval lo .. hi (inclusive).
     maxwidth: int
-        the maximum number of parallel edges
+        maximum number of parallel edges
 
     """
     explorer = BreadthFirstIterator(roots)
@@ -284,9 +329,16 @@ def count_tree_between(roots, lo, hi):
 
 
 def find_nodes_before(root, value):
-    """Identify all nodes that have children above value.
+    """Identify all nodes that have children above *value*.
 
-    If a root child is above the value, its parent (root) is the leaf returned.
+    If a root child is above the value, its parent (*root*) is the leaf returned.
+
+    Parameters
+    ----------
+    root: :pyclass:TreeNode
+        tree
+    value: float
+        selection threshold
 
     Returns
     --------
@@ -332,7 +384,7 @@ def find_nodes_before(root, value):
 class PointPile(object):
     """A in-memory linearized storage of point coordinates.
 
-    TreeNodes only store the logL value and id,
+    :pyclass:TreeNodes only store the logL value and id,
     which is the index in the point pile. The point pile stores
     the point coordinates.
     """
@@ -358,7 +410,20 @@ class PointPile(object):
         self.pdim = pdim
 
     def add(self, newpointu, newpointp):
-        """Save point `newpointu` (cube) / `newpointp` (parameters)."""
+        """Save point.
+
+        Parameters
+        -----------
+        newpointu: array
+            point (in u-space)
+        newpointp: array
+            point (in p-space)
+
+        Returns
+        ---------
+        index: int
+            index of the new point in the pile
+        """
         if self.nrows >= self.us.shape[0]:
             self.us = np.concatenate((self.us, np.zeros((self.chunksize, self.udim))))
             self.ps = np.concatenate((self.ps, np.zeros((self.chunksize, self.pdim))))
@@ -378,7 +443,22 @@ class PointPile(object):
         return self.ps[i]
 
     def make_node(self, value, u, p):
-        """Save point `u` (cube) / `p` (parameters), return a tree node."""
+        """Store point in pile, and create a new tree node that points to it.
+
+        Parameters
+        -----------
+        value: float
+            value to store in node (loglikelihood)
+        u: array
+            point (in u-space)
+        p: array
+            point (in p-space)
+
+        Returns
+        ---------
+        node: :pyclass:TreeNode
+            node
+        """
         index = self.add(u, p)
         return TreeNode(value=value, id=index)
 
@@ -540,8 +620,8 @@ class MultiCounter(object):
         self.ncounters = len(self.rootids)
 
         self.check_insertion_order = check_insertion_order
-        self.insertion_order_threshold = 2
-        self.insertion_order_accumulator = [UniformOrderAccumulator(self.rootids.shape[1]) for _ in range(self.rootids.shape[0])]
+        self.insertion_order_threshold = 4
+        self.insertion_order_accumulator = UniformOrderAccumulator()
 
         self.reset(len(self.rootids))
 
@@ -563,8 +643,8 @@ class MultiCounter(object):
         self.remainder_ratio = 1.0
         self.remainder_fraction = 1.0
 
-        [acc.reset() for acc in self.insertion_order_accumulator]
-        self.insertion_order_runs = [[] for _ in range(nentries)]
+        self.insertion_order_accumulator.reset()
+        self.insertion_order_runs = []
 
     @property
     def logZ_bs(self):
@@ -578,17 +658,57 @@ class MultiCounter(object):
 
     @property
     def insertion_order_runlength(self):
-        shortest_runs = []
-        for runs in self.insertion_order_runs[:1]:
-            if len(runs) == 0:
-                shortest_runs.append(np.inf)
-            else:
-                shortest_runs.append(min(runs))
-        return np.median(shortest_runs)
+        """Get shortest insertion order test run.
+
+        Returns
+        --------
+        shortest_run_length: int
+            Shortest insertion order test run length.
+
+        The MWW (U-test) statistic is considered at each iteration.
+        When it exceeds a threshold (4 sigma by default, `insertion_order_threshold`),
+        the statistic is reset. The run length is recorded.
+        This property returns the shortest run length of all recorded
+        so far, or infinity otherwise.
+
+        At 4 sigma, run lengths no shorter than 10^5.5 are expected
+        in unbiased runs.
+        """
+        runs = self.insertion_order_runs
+        if len(runs) == 0:
+            return np.inf
+        else:
+            return min(runs)
 
     @property
     def insertion_order_converged(self):
-        return len(self.insertion_order_runs[0]) == 0
+        """Check convergence.
+
+        Returns
+        --------
+        converged: bool
+            Whether the run is unbiased according to a U-test.
+
+        The MWW (U-test) statistic is considered at each iteration.
+        When it exceeds a threshold (4 sigma by default, `insertion_order_threshold`),
+        the statistic is reset. The run length is recorded.
+        This property returns the shortest run length of all recorded
+        so far, or infinity otherwise.
+
+        At 4 sigma, run lengths no shorter than 10^5.5 are expected
+        in unbiased runs. If the number of runs exceeds the number
+        of iterations divided by 10^5.5, the run is likely biased
+        and not converged.
+
+        If not converged, the step sampler may need to use more steps,
+        or the problem needs to be  reparametrized.
+        """
+        # we expect run lengths not shorter than 300000 for 4sigma
+        # if we get many more than expected from the number of iterations
+        # the run has not converged
+        niter = len(self.logweights)
+        expected_number = max(1, int(np.ceil(niter / 10**(5.5))))
+        return len(self.insertion_order_runs) <= expected_number
 
     def passing_node(self, rootid, node, rootids, parallel_values):
         """Accumulate node to the integration.
@@ -599,9 +719,9 @@ class MultiCounter(object):
 
         Parameters
         ----------
-        rootid: TreeNode
+        rootid: :pyclass:TreeNode
             root node this `node` is from.
-        node: TreeNode
+        node: :pyclass:TreeNode
             node being processed.
         rootids: array of ints
             for each parallel node, which root it belongs to.
@@ -682,19 +802,16 @@ class MultiCounter(object):
             self.logVolremaining = self.all_logVolremaining[0]
 
             if self.check_insertion_order and len(np.unique(parallel_values)) == len(parallel_values):
-                order_max = nlive.max() + 1
-                for i, acc in enumerate(self.insertion_order_accumulator):
-                    if not active[i]:
-                        continue
-                    parallel_values_here = parallel_values[self.rootids[i, rootids]]
-                    for child in node.children:
-                        # rootids is 400 ints pointing to the root id where each parallel_values is from
-                        # self.rootids[i] says which rootids belong to this bootstrap
-                        # need which of the parallel_values are active here
-                        acc.add((parallel_values_here < child.value).sum(), nlive[i])
-                        if abs(acc.zscore) > self.insertion_order_threshold:
-                            self.insertion_order_runs[i].append(len(acc))
-                            acc.reset()
+                acc = self.insertion_order_accumulator
+                parallel_values_here = parallel_values[self.rootids[0, rootids]]
+                for child in node.children:
+                    # rootids is 400 ints pointing to the root id where each parallel_values is from
+                    # self.rootids[i] says which rootids belong to this bootstrap
+                    # need which of the parallel_values are active here
+                    acc.add((parallel_values_here < child.value).sum(), nlive0)
+                    if abs(acc.zscore) > self.insertion_order_threshold:
+                        self.insertion_order_runs.append(len(acc))
+                        acc.reset()
 
         else:
             # contracting!
@@ -731,8 +848,36 @@ class MultiCounter(object):
 
 
 def combine_results(saved_logl, saved_nodeids, pointpile, main_iterator, mpi_comm=None):
-    """Combine a sequence of likelihoods and nodes into a summary dictionary."""
+    """Combine a sequence of likelihoods and nodes into a summary dictionary.
 
+    Parameters
+    ----------
+    saved_logl: list of floats
+        loglikelihoods of dead points
+    saved_nodeids: list of ints
+        indices of dead points
+    pointpile: :pyclass:PointPile
+        Point pile.
+    main_iterator: :pyclass:BreadthFirstIterator
+        iterator used
+    mpi_comm:
+        MPI communicator object, or None if MPI is not used.
+
+    Returns
+    --------
+    results: dict
+        All information of the run. Important keys:
+        Number of nested sampling iterations (niter),
+        Evidence estimate (logz),
+        Effective Sample Size (ess),
+        H (information gain),
+        weighted samples (weighted_samples),
+        equally weighted samples (samples),
+        best-fit point information (maximum_likelihood),
+        posterior summaries (posterior).
+        The rank order test score (insertion_order_MWW_test) is
+        included if the iterator has it.
+    """
     assert np.shape(main_iterator.logweights) == (len(saved_logl), len(main_iterator.all_logZ)), (
         np.shape(main_iterator.logweights),
         np.shape(saved_logl),
@@ -823,6 +968,40 @@ def logz_sequence(root, pointpile, nbootstraps=12, random=True, onNode=None, ver
     """Run MultiCounter through tree `root`.
 
     Keeps track of, and returns ``(logz, logzerr, logv, nlive)``.
+
+    Parameters
+    ----------
+    root: :pyclass:TreeNode
+        Tree
+    pointpile: :pyclass:PointPile
+        Point pile
+    nbootstraps: int
+        Number of independent iterators
+    random: bool
+        Whether to randomly draw volume estimates
+    onNode: function
+        Function to call for every node.
+        receives current node and the iterator
+    verbose: bool
+        Whether to show a progress indicator on stderr
+    check_insertion_order: bool
+        Whether to perform a rolling insertion order rank test
+
+    Returns
+    --------
+    results: dict
+        Run information, see :pyfunc:combine_results
+    sequence: dict
+        Each entry of the dictionary is results['niter'] long,
+        and contains the state of information at that iteration.
+        Important keys are:
+        Iteration number (niter),
+        Volume estimate (logvol), loglikelihood (logl), absolute logarithmic weight (logwt),
+        Relative weight (weights), point (samples),
+        Number of live points (nlive),
+        Evidence estimate (logz) and its uncertainty (logzerr),
+        Rank test score (insert_order).
+
     """
     roots = root.children
 
@@ -861,7 +1040,6 @@ def logz_sequence(root, pointpile, nbootstraps=12, random=True, onNode=None, ver
             # first time they are all the same
             logzerr.append(main_iterator.logZerr_bs)
 
-        insert_order_value = 0.0
         if len(np.unique(active_values)) == len(active_values) and len(node.children) > 0:
             child_insertion_order = (active_values > node.children[0].value).sum()
             insert_order.append(2 * (child_insertion_order + 1.) / len(active_values))
